@@ -6,37 +6,53 @@ use strict;
 use constant FIRST_PROPERTY => -1;
 # used to find the first property in the object
 
-use Games::Rezrov::MethodMaker ([],
-			 qw(
-			    size_byte
-			    search_id
-			    pointer
-			    property_offset
-			    property_exists
-			    property_number
-			    property_len
-			    zobj
-			    pre_v4
-			   )
-			 );
-
 use Games::Rezrov::Inliner;
+use Games::Rezrov::InlinedPrivateMethod;
 
-use SelfLoader;
+my $code = new Games::Rezrov::InlinedPrivateMethod("-manual" => 1,
+						   "-names" =>
+						   [ qw (
+							 _property_exists
+							 _property_number
+							 _property_len
+							 _property_offset
+							 _size_byte
+							 _pointer
+							 _pre_v4
+							 _zobj
+							 _search_id
+							)
+						   ],
+						  );
+Games::Rezrov::Inliner::inline($code);
 
-my $INLINE_CODE = '
+#print $$code; die;
+
+eval $$code; die $@ if $@;
+
+1;
+
+__DATA__
+
+sub property_exists {
+  # public, read-only
+  return $_[0]->_property_exists();
+}
+
+sub property_number {
+  # public, read-only
+  return $_[0]->_property_number();
+}
 
 sub get_value {
   # return this value for this property
-  if ($_[0]->property_exists()) {
+  if ($_[0]->_property_exists()) {
     # this object provides this property
-    my $len = $_[0]->property_len();
-    my $po = $_[0]->property_offset();
-#    print STDERR "get_value at $po\n";
+    my $len = $_[0]->_property_len();
     if ($len == 2) {
-      return GET_WORD_AT($po);
+      return GET_WORD_AT($_[0]->_property_offset());
     } elsif ($len == 1) {
-      return GET_BYTE_AT($po);
+      return GET_BYTE_AT($_[0]->_property_offset());
     } else {
       die "get_value() called on long property";
     }
@@ -50,8 +66,8 @@ sub next {
   # search for a specific property, or move to the next one
   my ($self, $search_id) = @_;
   die("attempt to read past end of property list")
-    if ($self->size_byte() == 0);
-  my $pointer = $self->pointer();
+    if ($self->_size_byte() == 0);
+  my $pointer = $self->_pointer();
 
   my $property_number;
   my $exists = 0;
@@ -59,7 +75,7 @@ sub next {
   my $property_len;
   my $last_id;
   my $property_offset = 0;
-  my $pre_v4 = $self->pre_v4();
+  my $pre_v4 = $self->_pre_v4();
   while (1) {
 #    print STDERR "search\n";
     $size_byte = GET_BYTE_AT($pointer);
@@ -112,7 +128,6 @@ sub next {
 	last;
       } else {
 	$last_id = $property_number;
-	#    printf STDERR "prop search %s: %d\n", $zobj->object_id, $self->property_number;
 	if ($property_number == $search_id) {
 	  #      print STDERR "got it\n";
 	  $exists = 1;
@@ -122,48 +137,39 @@ sub next {
       }
     }
   }
-  $self->property_exists($exists);
+  $self->_property_exists($exists);
 #  print STDERR "exists: $exists\n";
-  $self->property_len($property_len);
-  $self->property_number($property_number);
-  $self->size_byte($size_byte);
-  $self->property_offset($property_offset);
-  $self->pointer($pointer);
+  $self->_property_len($property_len);
+  $self->_property_number($property_number);
+  $self->_size_byte($size_byte);
+  $self->_property_offset($property_offset);
+  $self->_pointer($pointer);
 }
 
 sub get_default_value {
   # get the default value for this property ID
   # spec 12.2
   my $offset = Games::Rezrov::StoryFile::header()->object_table_address() +
-    (($_[0]->search_id() - 1) * 2);
+    (($_[0]->_search_id() - 1) * 2);
   # FIX ME
   return(GET_WORD_AT($offset));
 }
 
-';
-
-Games::Rezrov::Inliner::inline(\$INLINE_CODE);
-eval $INLINE_CODE;
-undef $INLINE_CODE;
-
-1;
-__DATA__
-
 sub new {
-  my ($type, $search_id, $zobj) = @_;
+  my ($type, $search_id, $zobj, $psi) = @_;
 
 #  printf STDERR "new zprop %s for obj %s\n", $search_id, $zobj->object_id();
 
   my $self = [];
   bless $self, $type;
 
-  $self->zobj($zobj);
-  $self->pre_v4(Games::Rezrov::StoryFile::version() <= 3);
-  $self->search_id($search_id);
+  $self->_zobj($zobj);
+  $self->_pre_v4(Games::Rezrov::StoryFile::version() <= 3);
+  $self->_search_id($search_id);
 
-  $self->size_byte(-1);
-  $self->pointer($zobj->property_start_index());
-  $self->property_offset(-1);
+  $self->_size_byte(-1);
+  $self->_pointer($psi);
+  $self->_property_offset(-1);
   $self->next($search_id);
   return $self;
 }
@@ -171,15 +177,15 @@ sub new {
 sub set_value {
   # set this property to specified value
   my ($self, $value) = @_;
-  if ($self->property_exists()) {
+  if ($self->_property_exists()) {
 #    print STDERR "set_value to $value\n";
-    my $len = $self->property_len();
-    my $offset = $self->property_offset();
+    my $len = $self->_property_len();
+    my $offset = $self->_property_offset();
     if (Games::Rezrov::ZOptions::SNOOP_PROPERTIES()) {
       Games::Rezrov::StoryFile::write_text(sprintf("[set value of property %d of %s (%s) = %d]",
-					   $self->property_number(),
-					   $self->zobj()->object_id(),
-					   ${$self->zobj()->print()},
+					   $self->_property_number(),
+					   $self->_zobj()->object_id(),
+					   ${$self->_zobj()->print()},
                                            $value), 1);
     }
     if ($len == 1) {
@@ -195,7 +201,7 @@ sub set_value {
 }
 
 sub get_data_address {
-  return $_[0]->property_offset();
+  return $_[0]->_property_offset();
 }
 
 sub get_next {
